@@ -207,6 +207,22 @@ slice_coerce_index(PyObject *o, npy_intp *v)
     return 1;
 }
 
+/*
+ * Same (slightly weird) semantics as slice_coerce_index, but raising a useful
+ * exception on failure. Written this way to get better error messages in
+ * 1.11, while creating minimal risky changes during beta cycle.
+ */
+static int
+slice_coerce_index_or_raise(PyObject *o, npy_intp *v)
+{
+    if (!slice_coerce_index(o, v)) {
+        PyErr_Format(PyExc_IndexError,
+                     "failed to coerce slice entry of type %s to integer",
+                     o->ob_type->tp_name);
+        return 0;
+    }
+    return 1;
+}
 
 /*
  * This is basically PySlice_GetIndicesEx, but with our coercion
@@ -226,7 +242,7 @@ slice_GetIndices(PySliceObject *r, npy_intp length,
         *step = 1;
     }
     else {
-        if (!slice_coerce_index(r->step, step)) {
+        if (!slice_coerce_index_or_raise(r->step, step)) {
             return -1;
         }
         if (*step == 0) {
@@ -241,7 +257,7 @@ slice_GetIndices(PySliceObject *r, npy_intp length,
         *start = *step < 0 ? length-1 : 0;
     }
     else {
-        if (!slice_coerce_index(r->start, start)) {
+        if (!slice_coerce_index_or_raise(r->start, start)) {
             return -1;
         }
         if (*start < 0) {
@@ -259,7 +275,7 @@ slice_GetIndices(PySliceObject *r, npy_intp length,
         *stop = defstop;
     }
     else {
-        if (!slice_coerce_index(r->stop, stop)) {
+        if (!slice_coerce_index_or_raise(r->stop, stop)) {
             return -1;
         }
         if (*stop < 0) {
@@ -1456,10 +1472,10 @@ PyArray_MultiIterFromObjects(PyObject **mps, int n, int nadd, ...)
     int i, ntot, err=0;
 
     ntot = n + nadd;
-    if (ntot < 2 || ntot > NPY_MAXARGS) {
+    if (ntot < 1 || ntot > NPY_MAXARGS) {
         PyErr_Format(PyExc_ValueError,
-                     "Need between 2 and (%d) "                 \
-                     "array objects (inclusive).", NPY_MAXARGS);
+                     "Need at least 1 and at most %d "
+                     "array objects.", NPY_MAXARGS);
         return NULL;
     }
     multi = PyArray_malloc(sizeof(PyArrayMultiIterObject));
@@ -1522,10 +1538,10 @@ PyArray_MultiIterNew(int n, ...)
 
     int i, err = 0;
 
-    if (n < 2 || n > NPY_MAXARGS) {
+    if (n < 1 || n > NPY_MAXARGS) {
         PyErr_Format(PyExc_ValueError,
-                     "Need between 2 and (%d) "                 \
-                     "array objects (inclusive).", NPY_MAXARGS);
+                     "Need at least 1 and at most %d "
+                     "array objects.", NPY_MAXARGS);
         return NULL;
     }
 
@@ -1603,12 +1619,12 @@ arraymultiter_new(PyTypeObject *NPY_UNUSED(subtype), PyObject *args, PyObject *k
             ++n;
         }
     }
-    if (n < 2 || n > NPY_MAXARGS) {
+    if (n < 1 || n > NPY_MAXARGS) {
         if (PyErr_Occurred()) {
             return NULL;
         }
         PyErr_Format(PyExc_ValueError,
-                     "Need at least two and fewer than (%d) "
+                     "Need at least 1 and at most %d "
                      "array objects.", NPY_MAXARGS);
         return NULL;
     }
